/****************************************************************************
*  widget_python.h  -  Functions for widget python api
*
*  Copyright (C) 2003 Hans Karlsson <karlsson.h@home.se>
*  Copyright (C) 2003-2004 Adam Geitgey <adam@rootnode.org>
*  Copyright (c) 2004 Petri Damstn <damu@iki.fi>
*
*  This file is part of Superkaramba.
*
*  Superkaramba is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  Superkaramba is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with Superkaramba; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/

#include "widget_python.h"
#include <qobject.h>
#include "karamba.h"
#include "meter.h"
#include "meter_python.h"

/* now a method we need to expose to Python */
int getWidgetXCoordinate(long widget)
{
  karamba* currTheme = (karamba*)widget;
  return currTheme->x();
}

/* now a method we need to expose to Python */
int getWidgetYCoordinate(long widget)
{
  karamba* currTheme = (karamba*)widget;
  return currTheme->y();
}

PyObject* py_get_widget_position(PyObject *self, PyObject *args)
{
  long widget;
  if(!PyArg_ParseTuple(args, "l:getWidgetPosition", &widget))
    return NULL;
  if (!checkKaramba(widget))
    return NULL;
  return Py_BuildValue("(i,i)", getWidgetXCoordinate(widget),
                                getWidgetYCoordinate(widget));
}

/* now a method we need to expose to Python */
long createWidgetMask(long widget, char* path)
{
  karamba* currTheme = (karamba*)widget;

  currTheme->clearMask();

  if (currTheme->widgetMask != 0)
  {
    delete currTheme->widgetMask;
  }

  QString maskpath;
  maskpath.setAscii(path);

  currTheme->widgetMask = new QBitmap(path);

  currTheme->kroot->stop();
  delete currTheme->kroot;

  //  currTheme->setMask(*(currTheme->widgetMask));

  QBitmap blah = QBitmap("/home/ageitgey/mask.png");
  currTheme->setMask(blah);


  currTheme->kroot = new KarambaRootPixmap((QWidget*)currTheme);
  currTheme->kroot->start();

  return (int)currTheme->widgetMask;
}

PyObject* py_create_widget_mask(PyObject *self, PyObject *args)
{
  long widget;
  char *text;
  if (!PyArg_ParseTuple(args, "ls:createWidgetMask", &widget, &text))
    return NULL;
  if (!checkKaramba(widget))
    return NULL;
  return Py_BuildValue("l", createWidgetMask(widget, text));
}

/* now a method we need to expose to Python */
long redrawWidgetBackground(long widget)
{
  karamba* currTheme = (karamba*)widget;
  currTheme->kroot->repaint(true);
  return 1;
}

PyObject* py_redraw_widget_background(PyObject *self, PyObject *args)
{
  long widget;
  if (!PyArg_ParseTuple(args, "l:redrawWidgetBackground", &widget))
    return NULL;
  if (!checkKaramba(widget))
    return NULL;
  return Py_BuildValue("l", redrawWidgetBackground(widget));
}

/* now a method we need to expose to Python */
long redrawWidget(long widget)
{
  karamba* currTheme = (karamba*)widget;
  currTheme->externalStep();
  return 1;
}

PyObject* py_redraw_widget(PyObject *self, PyObject *args)
{
  long widget;
  if (!PyArg_ParseTuple(args, "l:redrawWidget", &widget))
    return NULL;
  if (!checkKaramba(widget))
    return NULL;
  return Py_BuildValue("l", redrawWidget(widget));
}

/* now a method we need to expose to Python */
long resizeWidget(long widget, long x, long y)
{
  karamba* currTheme = (karamba*)widget;
  //currTheme->test = true;
  currTheme->setFixedSize((int)x,(int)y);
  //currTheme->test = false;
  return 1;
}

PyObject* py_resize_widget(PyObject *self, PyObject *args)
{
  long widget, x, y;
  if (!PyArg_ParseTuple(args, "lll:resizeWidget", &widget, &x, &y))
    return NULL;
  if (!checkKaramba(widget))
    return NULL;
  return Py_BuildValue("l", resizeWidget(widget, x, y));
}

/* now a method we need to expose to Python */
long moveWidget(long widget, long x, long y)
{
  karamba* currTheme = (karamba*)widget;
  currTheme->move((int)x, (int)y);
  return 1;
}

PyObject* py_move_widget(PyObject *self, PyObject *args)
{
  long widget, x, y;
  if (!PyArg_ParseTuple(args, "lll:moveWidget", &widget, &x, &y))
    return NULL;
  if (!checkKaramba(widget))
    return NULL;
  return Py_BuildValue("l", moveWidget(widget, x, y));
}

/* now a method we need to expose to Python */
long toggleWidgetRedraw(long widget, bool b)
{
  karamba* currTheme = (karamba*)widget;
  if (currTheme != 0)
  {
    currTheme->toggleWidgetUpdate( b );
  }
}

PyObject* py_toggle_widget_redraw(PyObject *self, PyObject *args)
{
  long widget, b;

  if (!PyArg_ParseTuple(args, "ll:toggleWidgetRedraw", &widget, &b ))
    return NULL;
  if (!checkKaramba(widget))
    return NULL;
  return Py_BuildValue("l", toggleWidgetRedraw(widget, b));
}
